#include <avr/io.h>
#include <avr/interrupt.h>

#define MAN_H_ID	(0x41)						//ESTA manufacturer ID
#define MAN_L_ID	(0x64)
#define TYPE_ID		(0x01)						//product ID 

struct RDM_Packet	{
	uint8_t  ASC;								//alternate start code
	uint8_t  SSC;								//sub start code
	uint8_t  Length;							//total packet size
	uint8_t  DestID[6];							//target address
	uint8_t  SrcID[6];				    		//source address
	uint8_t  TransNo;							//transaction number
	uint8_t  PortID;							//Port ID / Transaction Number
	uint8_t  MsgCount;							//message count
	uint16_t SubDev;							//sub device number (root = 0) 
	uint8_t  Cmd;								//command class
	uint16_t PID;								//parameter ID
	uint8_t  PDLen;								//parameter data length in bytes
	uint8_t  Data[30];							//data byte field (limit to 28, as last 2 bytes are used for check sum!!)
};



enum {EVAL_DMX, EVAL_RDM, MUTE_RDM};			//main flags

//DmxField in 'globals.h'
		 uint8_t	 RdmIdentifyCnt;			//counter for device identification - do something in main if >0 and decrement slowly
volatile uint8_t	 RdmField[60];				//array of RDM control data (raw)



// RDM CODES (ANSI E1.20)

//command classes
#define DISC_CMD				(0x10)
#define DISC_CMD_RESPONSE		(0x11)
#define GET_CMD					(0x20)
#define GET_CMD_RESPONSE		(0x21)
#define SET_CMD					(0x30)
#define SET_CMD_RESPONSE		(0x31)

// command class defines ??
#define CC_GET					(0x01)
#define CC_SET					(0x02)
#define CC_GET_SET				(0x03)

// data type definitions
#define DS_NOT_DEFINED			(0x00)
#define DS_BIT_FIELD			(0x01)
#define DS_UNSIGNED_BYTE		(0x03)
#define DS_SIGNED_BYTE			(0x04)
#define DS_UNSIGNED_WORD		(0x05)
#define DS_SIGNED_WORD			(0x06)

//response type
#define RESPONSE_TYPE_ACK		(0x00)
#define RESPONSE_TYPE_NACK		(0x02)	//with reason!!

//NACK reasons (MSB is always zero - left out!!)
#define NR_UNKNOWN_PID			(0x00)
#define NR_FORMAT_ERROR			(0x01)
#define NR_HARDWARE_FAULT		(0x02)
#define NR_WRITE_PROTECT		(0x04)
#define NR_UNSUPPORTED_CC		(0x05)
#define NR_DATA_OUT_OF_RANGE	(0x06)
#define NR_BUFFER_FULL			(0x07)
#define NR_PACKET_SIZE_UNSUPPORTED (0x08)
#define NR_NO_SUBDEVICE			(0x09)

//device status
#define STATUS_NONE				(0x00)
#define STATUS_WARNING		    (0x03)
#define STATUS_ERROR			(0x04)


//status msg IDs (error types),(MSB is zero and left out...)
#define STS_CAL_FAIL			(0x01)	//slot label
#define STS_SENS_NOT_FOUND		(0x02)	//"
#define STS_SENS_ALWAYS_ON		(0x03)	//"
#define STS_LAMP_DOUSED			(0x11)
#define STS_LAMP_STRIKE			(0x12)
#define STS_OVERTEMP			(0x21)	//SensorID, Temp
#define STS_UNDERTEMP			(0x22)	//"
#define STS_SENS_OUT_RANGE		(0x23)	//sensorID
#define STS_OVERVOLTAGE_PHASE	(0x31)	//Phase, Voltage
#define STS_UNDERVOLTAGE_PHASE	(0x32)	//"
#define STS_OVERCURRENT			(0x33)	//Phase, Current
#define STS_PHASE_ERROR			(0x36)	//Phase
#define STS_BREAKER_TRIP		(0x42)
#define STS_DIM_FAILURE			(0x44)
#define STS_READY				(0x50)	//slot label
#define STS_NOT_READY			(0x51)	//"
#define STS_LOW_FLUID			(0x52)	//"


//parameter IDs !!bytes are swapped!!
#define DISC_UNIQUE_BRANCH		(0x0001)	//network management
#define DISC_MUTE				(0x0002)
#define DISC_UN_MUTE			(0x0003)
#define IDENTIFY				(0x1000)
#define SUPPORTED_PARAMETERS	(0x0050)	//information
#define PARAMETER_DESCRIPTION	(0x0051)
#define DEVICE_INFO				(0x0060)
#define DEVICE_LABEL			(0x0082)
#define MANUFACT_LABEL          (0x0081)
#define SOFTWARE_VERSION_LABEL  (0x00C0)
#define STATUS_MESSAGES			(0x0030)
#define DEVICE_HOURS            (0x0400)
#define LAMP_HOURS              (0x0401)
#define DMX_START_ADDRESS		(0x00F0)	//DMX setup
#define PERSONALITY				(0x00E0)
#define MIN_LIMIT				(0x0300)
#define MAX_LIMIT				(0x0301)
#define ERR_SCENE				(0x0302)
#define DIMMSWITCH				(0x0304)



//product category
#define FIXT_FIXED				(0x0101)	//not moving fixture
#define FIXT_DIMMER				(0x0501)	//dimmer pack
#define FIXT_DIM_LED			(0x0509)	//LED dimmer


extern void start_DMX_Tx(void);
extern void new_DMX_Tx(void);
extern void start_DMX_Rx(void);
extern void    check_rdm(void);
